package com.huimv.guowei.admin.controller;


import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.guowei.admin.common.utils.DataUill;
import com.huimv.guowei.admin.common.utils.PdfUtil;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.entity.EnergyElectricity;
import com.huimv.guowei.admin.entity.EnergyWarningInfo;
import com.huimv.guowei.admin.entity.EnergyWarningThreshold;
import com.huimv.guowei.admin.service.IEnergyElectricityService;
import com.huimv.guowei.admin.service.IEnergyWarningInfoService;
import com.huimv.guowei.admin.service.IEnergyWarningThresholdService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author author
 * @since 2023-06-01
 */
@RestController
@RequestMapping("/energy-electricity")
@CrossOrigin
public class EnergyElectricityController {
    @Resource
    private IEnergyElectricityService energyElectricityService;
    @Autowired
    private IEnergyWarningThresholdService thresholdService;
    @Autowired
    private IEnergyWarningInfoService warningInfoService;

    @PostMapping("/getElectricity")
    public Result getElectricity(@RequestBody Map<String ,String> paramsMap){
        return energyElectricityService.getElectricity(paramsMap);
    }

    @PostMapping("/listElectricity")
    public Result listElectricity(@RequestBody Map<String ,String> paramsMap){
        return energyElectricityService.listElectricity(paramsMap);
    }

    @PostMapping("/listUnitElectricity")
    public Result listUnitElectricity(@RequestBody Map<String ,String> paramsMap){
        return energyElectricityService.listUnitElectricity(paramsMap);
    }

    @PostMapping("/getScreenElectricity")
    public Result getScreenElectricity(@RequestBody Map<String ,String> paramsMap){
        return energyElectricityService.getScreenElectricity(paramsMap);
    }

    @PostMapping("/listScreenElectricity")
    public Result listScreenElectricity(@RequestBody Map<String ,String> paramsMap){
        return energyElectricityService.listScreenElectricity(paramsMap);
    }

    @GetMapping("/printElectricity")
    public void printDuck(HttpServletResponse response, @RequestParam(name = "type") String type,
                          @RequestParam(name = "farmId") String farmId,
                          @RequestParam(name = "startTime", required = false) String startTime,
                          @RequestParam(name = "endTime", required = false) String endTime) throws Exception {

        Map<String, String> map = new HashMap<>();
        map.put("type", type);
        map.put("farmId", farmId);
        if (startTime != null) {
            map.put("startTime", startTime);
            map.put("endTime", endTime);
        }
        energyElectricityService.printElectricity(response, map);
        String path = "/opt/guowei/electricity.xls";
        PdfUtil.returnPdfStream3(response, path, "用电信息");
    }

    //平养用电填报
    @PostMapping("/addElectricity")
    public Result addElectricity(@RequestBody Map<String ,String> paramsMap) throws ParseException {
        String farmId = paramsMap.get("farmId");
        String startDate = paramsMap.get("startDate");
        String endDate = paramsMap.get("endDate");
        String electricity = paramsMap.get("electricityValue");
        String unitId = paramsMap.get("unitId");
        String unitName = paramsMap.get("unitName");
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Date start = sdf.parse(startDate);
        Date end = sdf.parse(endDate);
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(start);
        List<Date> dateList = new ArrayList<>();
        while (calendar.getTime().before(end) || calendar.getTime().equals(end)) {
            dateList.add(calendar.getTime());
            calendar.add(Calendar.DATE, 1);
        }
        Double value = Double.valueOf(electricity);
        double v = value / dateList.size();
        DecimalFormat def = new DecimalFormat("0.00");
        for (Date date : dateList) {
            QueryWrapper<EnergyElectricity> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("chip_code", 1006).eq("unit_id", unitId).eq("create_date", date);
            EnergyElectricity one = energyElectricityService.getOne(queryWrapper);
            if (ObjectUtil.isNotEmpty(one)) {
                one.setElectricityValue(def.format(v));
                energyElectricityService.updateById(one);
            } else {
                EnergyElectricity electricity1 = new EnergyElectricity();
                electricity1.setFarmId(Integer.parseInt(farmId));
                electricity1.setElectricityValue(def.format(v));
                electricity1.setUnitId(Integer.parseInt(unitId));
                electricity1.setOther1(unitName);
                electricity1.setChipCode("1006");
                electricity1.setCreateDate(date);
                energyElectricityService.save(electricity1);
            }
            EnergyWarningThreshold threshold = thresholdService.getOne(new QueryWrapper<EnergyWarningThreshold>().eq("farm_id", farmId));
            if (ObjectUtil.isNotEmpty(threshold)) {
                String groupElectricity = threshold.getGroupElectricity();
                BigDecimal e1 = new BigDecimal(electricity);
                BigDecimal e2 = new BigDecimal(groupElectricity);
                if (e1.compareTo(e2) >= 0) {
                    EnergyWarningInfo warningInfo = new EnergyWarningInfo();
                    warningInfo.setWarningType(2);
                    warningInfo.setBuildLocation(unitName);
                    warningInfo.setUnitId(Integer.parseInt(unitId));
                    warningInfo.setChipId("1006");
                    if ("9942".equals(unitId)) {
                        warningInfo.setDeviceId(224);
                    } else {
                        warningInfo.setDeviceId(225);
                    }
                    warningInfo.setDate(date);
                    warningInfo.setWarningContent("当前用电量过高");
                    warningInfoService.save(warningInfo);
                }
            }
        }
        return new Result(ResultCode.SUCCESS);
    }

    @PostMapping("/groupElectricity")
    public Result groupElectricity(@RequestBody Map<String ,String> paramsMap){
        String farmId = paramsMap.get("farmId");
        String unitId = paramsMap.get("unitId");
        String type = paramsMap.get("type");
        QueryWrapper<EnergyElectricity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq("unit_id",unitId);
        if ("1".equals(type)) {
            Date timesmorning = DataUill.getTimesmorning();
            queryWrapper.ge("create_date", timesmorning);
        } else if ("2".equals(type)) {
            DateTime dateTime = DateUtil.beginOfWeek(new Date());
            queryWrapper.ge("create_date", dateTime);
        } else if ("3".equals(type)) {
            Date timesMonthmorning = DataUill.getTimesMonthmorning();
            queryWrapper.ge("create_date", timesMonthmorning);
        } else {
            String startTime = paramsMap.get("startDate");
            String endTime = paramsMap.get("endDate");
            queryWrapper.between("create_date", startTime + " 00:00:00", endTime + " 23:59:59");
        }
        return new Result(ResultCode.SUCCESS,energyElectricityService.list(queryWrapper));
    }
    //大屏的用电监测
    @PostMapping("/groupElectricityScreen")
    public Result groupElectricityScreen(@RequestBody Map<String ,String> paramsMap){
        String farmId = paramsMap.get("farmId");
        String unitId = paramsMap.get("unitId");
        QueryWrapper<EnergyElectricity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq("unit_id",unitId);
        queryWrapper.orderByDesc("create_date").last(" limit 7");
        List<EnergyElectricity> list = energyElectricityService.list(queryWrapper);
        List<EnergyElectricity> list1 = new ArrayList<>();
        if (list.size() != 0) {
            for (int i = 0; i < list.size(); i++) {
                list1.add(list.get(list.size() - i-1));
            }
        }
        return new Result(ResultCode.SUCCESS,list1);
    }

    @PostMapping("/groupEditElectricity")
    public Result groupEditElectricity(@RequestBody EnergyElectricity energyElectricity){
        energyElectricityService.updateById(energyElectricity);
        return new Result(ResultCode.SUCCESS);
    }

    @PostMapping("/groupDeleteElectricity")
    public Result groupDeleteElectricity(@RequestBody Map<String ,String> paramsMap){
        energyElectricityService.removeById(paramsMap.get("id"));
        return new Result(ResultCode.SUCCESS);
    }

    @PostMapping("/groupListElectricity")
    public Result groupListElectricity(@RequestBody Map<String ,String> paramsMap){
        String farmId = paramsMap.get("farmId");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        Page<EnergyElectricity> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        QueryWrapper<EnergyElectricity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).in("unit_id", 9942, 9943).orderByDesc("create_date");
        return new Result(ResultCode.SUCCESS, energyElectricityService.page(page, queryWrapper));
    }
}
