package com.huimv.guowei.admin.group.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.guowei.admin.common.utils.DataUill;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.group.entity.GroupDuckInfo;
import com.huimv.guowei.admin.group.entity.GroupEating;
import com.huimv.guowei.admin.group.entity.GroupRank;
import com.huimv.guowei.admin.group.entity.vo.BreedEatingEgg;
import com.huimv.guowei.admin.group.entity.vo.GroupEatingVo;
import com.huimv.guowei.admin.group.mapper.GroupDuckInfoMapper;
import com.huimv.guowei.admin.group.mapper.GroupEatingMapper;
import com.huimv.guowei.admin.group.mapper.GroupRankMapper;
import com.huimv.guowei.admin.group.service.IGroupEatingService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 平养采食记录 服务实现类
 * </p>
 *
 * @author author
 * @since 2024-09-25
 */
@Service
public class GroupEatingServiceImpl extends ServiceImpl<GroupEatingMapper, GroupEating> implements IGroupEatingService {

    @Autowired
    private GroupEatingMapper eatingMapper;
    @Autowired
    private GroupRankMapper rankMapper;
    @Autowired
    private GroupDuckInfoMapper infoMapper;
    @Override
    public Result listPage(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        if ("".equals(pageNum) || null == pageNum) {
            pageNum = "1";
            pageSize = "20";
        }
        String keyword = paramsMap.get("keyword");
        String time = paramsMap.get("time");
        String id = paramsMap.get("id");
        Page<GroupEating> page = new Page(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        QueryWrapper<GroupEating> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).orderByDesc("eating_date");
        if (StringUtils.checkValNotNull(id)) {
            GroupDuckInfo duckInfo = infoMapper.selectById(id);
            queryWrapper.eq("farm_id", farmId).orderByDesc("eating_date");
            queryWrapper.eq("device_code", keyword).eq( "batch_num", duckInfo.getBatchNum());
        }else {
            if (StringUtils.isNotBlank(keyword)) {
                queryWrapper.and(wrapper -> wrapper.like(StringUtils.isNotBlank(keyword), "device_code", keyword)
                        .or().like(StringUtils.isNotBlank(keyword), "batch_num", keyword));
            }
        }
        queryWrapper.between(StringUtils.isNotBlank(time), "eating_date", time + " 00:00:00", time + " 23:59:59");
        return new Result(ResultCode.SUCCESS, eatingMapper.selectPage(page, queryWrapper));
    }

    @Override
    public Result listDetail(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String deviceCode = paramsMap.get("id");
        String batchNum = paramsMap.get("batchNum");
        QueryWrapper<GroupEating> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq("device_code", deviceCode)
                .eq(StringUtils.isNotBlank(batchNum), "batch_num", batchNum);
        return new Result(ResultCode.SUCCESS, eatingMapper.listDetail(queryWrapper));
    }

    @Override
    public Result BreedingEating(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String deviceCode = paramsMap.get("deviceCode");
        String batchNum = paramsMap.get("batchNum");
        String type = paramsMap.get("type");
        if ("".equals(type) || null == type) {
            type = "2";
        }
        QueryWrapper<GroupEating> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq("device_code", deviceCode).eq("batch_num", batchNum);
        if ("1".equals(type)) {
            Date timesmorning = DataUill.getTimesmorning();
            queryWrapper.ge("eating_date", timesmorning);
            return new Result(ResultCode.SUCCESS, eatingMapper.selectList(queryWrapper));
        } else if ("2".equals(type)) {
            DateTime beginOfWeek = DateUtil.beginOfWeek(new Date());
            queryWrapper.ge("eating_date", beginOfWeek);
            return new Result(ResultCode.SUCCESS, eatingMapper.breedList(queryWrapper));
        } else if ("3".equals(type)) {
            Date monthmorning = DataUill.getTimesMonthmorning();
            queryWrapper.ge("eating_date", monthmorning);
            return new Result(ResultCode.SUCCESS, eatingMapper.breedList(queryWrapper));
        } else {
            String startTime = paramsMap.get("startDate");
            String endTime = paramsMap.get("endDate");
            queryWrapper.between("eating_date", startTime+" 00:00:00", endTime+" 23:59:59");
            return new Result(ResultCode.SUCCESS, eatingMapper.selectList(queryWrapper));
        }
    }

    @Override
    public Result BreedingEatingEgg(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String deviceCode = paramsMap.get("deviceCode");
        String batchNum = paramsMap.get("batchNum");
        String type = paramsMap.get("type");
        if ("".equals(type) || null == type) {
            type = "1";
        }
        List<GroupRank> list = new ArrayList<>();
        QueryWrapper<GroupRank> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("device_code", deviceCode).eq("farm_id", farmId).eq("batch_num",batchNum);
        if ("1".equals(type)) {
            DateTime beginOfWeek = DateUtil.beginOfWeek(new Date());
            queryWrapper.ge("date_time", beginOfWeek);
             list = rankMapper.selectList(queryWrapper);
        } else if ("2".equals(type)) {
            Date monthmorning = DataUill.getTimesMonthmorning();
            queryWrapper.ge("date_time", monthmorning);
            list = rankMapper.selectList(queryWrapper);
        } else if ("3".equals(type)) {
            Date monthmorning = DateUtil.beginOfYear(new Date());
            queryWrapper.ge("date_time", monthmorning);
            list = rankMapper.getList(queryWrapper);
        } else {
            String startTime = paramsMap.get("startDate");
            String endTime = paramsMap.get("endDate");
            queryWrapper.between("date_time", startTime + " 00:00:00", endTime + " 23:59:59");
            list = rankMapper.selectList(queryWrapper);
        }
        return new Result(ResultCode.SUCCESS, list);
    }

    @Override
    public GroupEatingVo getEatingCount(String deviceCode,String batchNum) {
        return eatingMapper.getEating(deviceCode,batchNum);
    }

}
