package com.huimv.guowei.admin.group.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.guowei.admin.common.utils.Result;
import com.huimv.guowei.admin.common.utils.ResultCode;
import com.huimv.guowei.admin.group.entity.GroupBatch;
import com.huimv.guowei.admin.group.entity.GroupDuckInfo;
import com.huimv.guowei.admin.group.entity.GroupEating;
import com.huimv.guowei.admin.group.entity.GroupLeave;
import com.huimv.guowei.admin.group.mapper.GroupBatchMapper;
import com.huimv.guowei.admin.group.mapper.GroupDuckInfoMapper;
import com.huimv.guowei.admin.group.mapper.GroupLeaveMapper;
import com.huimv.guowei.admin.group.service.IGroupLeaveService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 平养离场记录 服务实现类
 * </p>
 *
 * @author author
 * @since 2024-09-25
 */
@Service
public class GroupLeaveServiceImpl extends ServiceImpl<GroupLeaveMapper, GroupLeave> implements IGroupLeaveService {

    @Autowired
    private GroupLeaveMapper leaveMapper;
    @Autowired
    private GroupBatchMapper batchMapper;
    @Autowired
    private GroupDuckInfoMapper duckInfoMapper;

    @Override
    public Result listPage(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        String type = paramsMap.get("type");
        if ("".equals(pageNum) || null == pageNum) {
            pageNum = "1";
            pageSize = "20";
        }
        String keyword = paramsMap.get("keyword");
        String time = paramsMap.get("time");
        Page<GroupLeave> page = new Page(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        QueryWrapper<GroupLeave> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).orderByDesc("leave_date").eq(StringUtils.isNotBlank(type), "type", type);
        if (StringUtils.isNotBlank(keyword)) {
            queryWrapper.and(wrapper -> wrapper.like(StringUtils.isNotBlank(keyword), "device_code", keyword)
                    .or().like(StringUtils.isNotBlank(keyword), "device_code", keyword));
        }
        queryWrapper.between(StringUtils.isNotBlank(time), "leave_date", time + " 00:00:00", time + " 23:59:59");
        queryWrapper.orderByDesc("create_time");
        return new Result(ResultCode.SUCCESS, leaveMapper.selectPage(page, queryWrapper));
    }

    @Override
    public Result add(GroupLeave leave) {
        if (leave.getType() == 0) {
            GroupDuckInfo groupDuckInfo = duckInfoMapper.selectOne(new QueryWrapper<GroupDuckInfo>().
                    eq("device_code", leave.getDeviceCode()).eq("is_leave", 0));
            leave.setBatchNum(groupDuckInfo.getBatchNum());
            leave.setCreateTime(new Date());
            String deviceCode = leave.getDeviceCode();
            QueryWrapper<GroupDuckInfo> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("farm_id", leave.getFarmId()).eq("device_code", deviceCode).eq("batch_num",groupDuckInfo.getBatchNum());
            GroupDuckInfo duckInfo = duckInfoMapper.selectOne(queryWrapper);
            duckInfo.setIsLeave(1);
            duckInfo.setCreateTime(new Date());
            duckInfoMapper.updateById(duckInfo);
        } else {
            leave.setBatchNum(leave.getDeviceCode());
            leave.setCreateTime(new Date());
            QueryWrapper<GroupBatch> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("farm_id", leave.getFarmId()).eq("batch_num", leave.getDeviceCode());
            GroupBatch groupBatch = batchMapper.selectOne(queryWrapper);
            String duckId = groupBatch.getDuckId();
            if (StringUtils.isNotBlank(duckId)) {
                String[] split = duckId.split(",");
                for (String muskId : split) {
                    GroupDuckInfo duckInfo = duckInfoMapper.selectOne(new QueryWrapper<GroupDuckInfo>().eq("device_code", muskId)
                            .eq("batch_num",leave.getDeviceCode()));
                    if (ObjectUtil.isNotEmpty(duckInfo)) {
                        duckInfo.setIsLeave(1);
                        duckInfo.setCreateTime(new Date());
                        duckInfoMapper.updateById(duckInfo);
                    }
                }
            }
            groupBatch.setDuckId("");
            groupBatch.setDisable(1);
            batchMapper.updateById(groupBatch);
        }
        return new Result(ResultCode.SUCCESS, leaveMapper.insert(leave));
    }

    @Override
    public Result edit(GroupLeave leave) {
        if (leave.getIsWithdraw() == 1) {
            if (leave.getType() == 0) {
                leave.setCreateTime(new Date());
                String deviceCode = leave.getDeviceCode();
                QueryWrapper<GroupDuckInfo> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("farm_id", leave.getFarmId()).eq("device_code", deviceCode).eq("is_leave",0);
                GroupDuckInfo duckInfo = duckInfoMapper.selectOne(queryWrapper);
                if (ObjectUtil.isNotEmpty(duckInfo)) {
                    return new Result(10001, "撤回失败，存在同批次同脚环的鸭只", false);
                } else {
                    GroupLeave groupLeave = leaveMapper.selectById(leave.getId());
                    String batchNum = groupLeave.getBatchNum();
                    GroupDuckInfo groupDuckInfo = duckInfoMapper.selectOne(new QueryWrapper<GroupDuckInfo>().eq("farm_id", leave.getFarmId())
                            .eq("device_code", deviceCode).eq("batch_num", batchNum));
                    groupDuckInfo.setIsLeave(0);
                    groupDuckInfo.setCreateTime(new Date());
                    duckInfoMapper.updateById(groupDuckInfo);
                }
            } else {
                leave.setCreateTime(new Date());
                QueryWrapper<GroupBatch> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("farm_id", leave.getFarmId()).eq("batch_num",leave.getDeviceCode());
                GroupBatch groupBatch = batchMapper.selectOne(queryWrapper);

                String duckIds = "";
                List<String> deviceCodes = new ArrayList<>();
                if (ObjectUtil.isNotEmpty(groupBatch)) {
                    String batchNum = groupBatch.getBatchNum();
                    List<GroupDuckInfo> groupDuckInfos = duckInfoMapper.selectList(new QueryWrapper<GroupDuckInfo>()
                            .eq("batch_num", batchNum).eq("farm_id",leave.getFarmId()));
                    groupDuckInfos.forEach(item->{
                        deviceCodes.add(item.getDeviceCode());
                    });
                    if (deviceCodes.size() != 0) {
                        Integer count = duckInfoMapper.selectCount(new QueryWrapper<GroupDuckInfo>().in("device_code", deviceCodes)
                                .eq("farm_id",leave.getFarmId()).eq("is_leave",0));
                        if (count != 0) {
                            return new Result(10001, "撤回失败，当前档案中存在相同脚环号鸭只！", false);
                        }
                    }
                    if (groupDuckInfos.size() != 0) {
                        for (int i = 0; i < groupDuckInfos.size(); i++) {
                            groupDuckInfos.get(i).setIsLeave(0);
                            groupDuckInfos.get(i).setCreateTime(new Date());
                            duckInfoMapper.updateById(groupDuckInfos.get(i));
                            if (i < groupDuckInfos.size() - 1) {
                                duckIds = groupDuckInfos.get(i).getDeviceCode() + ",";
                            } else {
                                duckIds = duckIds + groupDuckInfos.get(i).getDeviceCode();
                            }
                        }
                    }
                    leaveMapper.delete(new QueryWrapper<GroupLeave>().eq("farm_id", leave.getFarmId())
                            .eq("batch_num", groupBatch.getBatchNum()).eq("type", 0));
                }
                groupBatch.setDuckId(duckIds);
                groupBatch.setDisable(0);
                batchMapper.updateById(groupBatch);
            }
            leaveMapper.deleteById(leave.getId());
        } else {
            leaveMapper.updateById(leave);
        }
        return new Result(ResultCode.SUCCESS);
    }
}
